/***************************************************************************************************************************/
/*

Demo-Software for smart cameras from Vision Components
------------------------------------------------------

Program: Klaus Schneider,  VC


Program Description:
--------------------

This demofile gives the maximum speed of the camera. The image calculation is done in parallel
to the new image acquisition. The exposer time is done in parallel to the image transfer or 
calculation.

For example: 

1) Image transfer  = 18.5ms (VC2065), exposure time = 10ms, calculation time=10ms
   Inspection Time = 18.5ms or Frames per second = 54;

2) Image transfer  = 18.5ms (VC2065), exposer time = 10ms, calculation time=18.5ms
   Inspection Time = 18.5ms or Frames per second = 54;

3) Image transfer  = 18.5ms (VC2065), exposer time = 10ms, calculation time=30ms
   Inspection Time = 30ms or Frames per second = 33;

4) Image transfer  = 18.5ms (VC2065), exposer time = 10ms, calculation time=40ms
   Inspection Time = 40ms or Frames per second = 25;


At least you have to allocate 3 Screens: 

- one for Capture     (current Screen+2)
- one for Calculation (current Screen+1)
- one for Display     (current Screen  )


The function ScreenBuffer(int mode) works like this:

1) mode>0: 
   The function allocates the nr of Screens (mode=3: 3 Screens, mode=4: 4 Screens)
   Standard is mode=4. If the calculation time is higher than the image acquistion time
   (exposure + image transfer) you also take mode=4 or mode=3. 
   Minimum is 3 screens, maximum is 20 screens.

2) mode=0: 
   It switch the screens for Display, Calculation and Capture. If necessary it waits 
   for the image acquisition

3) mode<0:
   The funcition release the allocated screens and switch back to the original screen


The return values of the funcion are:

-1: the function couldn't allocate memory for all screens
-2: couldn't set a capture_request
-3: you want to switch screens (mode=0) or release memory (mode<0), but no screens are allocated 
-4: wrong nr of screens (have to be in range of [3,20])
-5: screens are already allocated (you call two times the function with mode>0)

*/
/***************************************************************************************************************************/

#include <vcrt.h>
#include <vclib.h>
#include <macros.h>
#include <sysvar.h>

/***************************************************************************************************************************/

#define  IniNrOfScreens      4  
#define  ReleaseAllocation  -1
#define  NextScreen          0

/***************************************************************************************************************************/

void calculation   (int time_ms);
int  ScreenBuffer  (int mode);

/***************************************************************************************************************************/

void main(void)

  {
  int   TimeAllMs, TimeCalcMs=100, result;
  char  key=0; 

    /*******************************************************************************/
    /*                 PLEASE INITIALIZE YOUR LICENCE CODES                        */
    /*                    To activate library functions                            */
    /*                                                                             */
    /* VCLIB :         init_licence("T1122334455") or init_licence("LT1122334455") */
    /* Extension Lib : init_licence("E1122334455") or init_licence("LE1122334455") */
    /* Color Lib :     init_licence("C1122334455") or init_licence("LC1122334455") */
    /* etc.                                                                        */
    /*                                                                             */
    /*      init_licence() returns 0 if licence code is OK, -5 otherwise           */
    /*                                                                             */
    /* Example :                                                                   */
    /*                                                                             */
	/**/ 	print("Init VCLIB = %d\n", init_licence("T1122334455"));			 /**/
    /*******************************************************************************/


  vmode(vmStill);

  /* ini Screen buffer (mode>0) */
  result = ScreenBuffer(IniNrOfScreens);
  if (result < 0) {print("Screen Allocation Error %d\n",result); return;}

  print("Press '+' to increase simulated calculation time\n"); 
  print("Press '-' to decrease simulated calculation time\n"); 
  print("Press 'q' to quit\n"); 
  print("Press any key to start\n"); 
  getchar();

  do
    {
    if (kbhit())
      {
      key = rs232rcv();

      if (key == '+') {TimeCalcMs+=10; if (TimeCalcMs> 500) TimeCalcMs=500;}
      if (key == '-') {TimeCalcMs-=10; if (TimeCalcMs<   0) TimeCalcMs=  0;}
      }

    /* measure time */
    TimeAllMs = getvar(MSEC);
 
    /* take next Screen (mode=0) */
    print("R=%d ",ScreenBuffer(NextScreen));

    /* simulate calculation (this function only waits 'TimeCalcMs' ms) */
    calculation(TimeCalcMs);

    /* calculation time */
    TimeAllMs = getvar(MSEC)-TimeAllMs; 
    if (TimeAllMs<0) TimeAllMs+=1000;

    print("InspectionTime=%dms  CalculationTime=%dms\n",TimeAllMs,TimeCalcMs);

    }
  while(key != 'q');

  /* release screen allocation (mode<0) */
  ScreenBuffer(ReleaseAllocation);
  }

/***************************************************************************************************************************/

void calculation (int time_ms)
  {
  /* waiting time in ms */
  time_delay(time_ms);

  /* 
  Please work here on the logical address
  for example:
  a) ScrGetLogPage (gives you the logical page where you can work) or
  b) ScrByteAddr(x,y) (calculates the address of x-y-position at the logical page)
  */
  }

/***************************************************************************************************************************/

int ScreenBuffer (int mode)
  {
  int i, exp=178, gain=0x60;
  static int NrOfScreens=0, *ScrAdr, *Screen, *TrackNr, disp, calc, capt;
  image  area;

  /* only possible in vmStill mode */
  vmode(vmStill);

  /* screen allocation mode */
  if (mode>0)
    {
    if (NrOfScreens != 0) return(-5);
    if (mode<3)  return(-4);
    if (mode>20) return(-4);
    
    NrOfScreens=mode;

    ScrAdr=(int *)sysmalloc(NrOfScreens,2); 
    if((int)ScrAdr==0) 
      {
      return(-1);
      }

    Screen=(int *)sysmalloc(NrOfScreens,2); 
    if((int)Screen==0) 
      {
      sysfree((void *)ScrAdr);
      return(-1);
      }

    TrackNr=(int *)sysmalloc(NrOfScreens,2); 
    if((int)TrackNr==0) 
      {
      sysfree((void *)Screen);
      sysfree((void *)ScrAdr);
      return(-1);
      }

    Screen[0]=ScrGetCaptPage;
    for (i=1; i<NrOfScreens; i++)  
      {
      ScrAdr[i]=(int)DRAMDisplayMalloc(); 
      if (ScrAdr[i]==0)
        {
        for (i-=1; i>0; i--)  
          {
          DRAMPgFree(ScrAdr[i]);
          }
        sysfree((void *)TrackNr);
        sysfree((void *)Screen);
        sysfree((void *)ScrAdr);
        return(-1);
        }

      /* Image display addresses have to be in alignments of 1024 */ 
      Screen[i]=(ScrAdr[i]&0xFFFFFC00) + 1024;

      /* delete outer areas of VC2028 */
      ImageAssign(&area, (long)Screen[i], DispGetColumns, DispGetRows, ScrGetPitch);  
      set(&area,0);
      }

    for (i=0; i<NrOfScreens-2; i++)  
      {
      TrackNr[i]=capture_request(exp,gain,(int *)Screen[i],0);
      if(TrackNr[i]==0) return(-2);
      }
    calc=0;
    disp=NrOfScreens-1;
    capt=NrOfScreens-2;
    }
    
  /* change screen for display, calculation and acquisition */
  if (mode==0)
    {
    if (NrOfScreens==0) return(-3);

    /* waiting for complete image */
    while(TrackNr[calc] > getvar(IMGREADY));

    TrackNr[capt]=capture_request(exp,gain,(int *)Screen[capt],0);
    if(TrackNr[capt]==0) return(-2);

    ScrSetDispPage(Screen[disp]);
    ScrSetLogPage (Screen[calc]);

    capt=disp;
    disp=calc;
    calc++;
    if (calc==NrOfScreens) calc=0;
    }


  /* release screen allocation mode */
  if (mode<0)
    {
    if (NrOfScreens==0) return(-3);
 
    ScrSetPhysPage(Screen[0]);

    for (i=NrOfScreens-1; i>0; i--)  
      {
      DRAMPgFree(ScrAdr[i]);
      }

    NrOfScreens=0;

    sysfree((void *)TrackNr);
    sysfree((void *)Screen);
    sysfree((void *)ScrAdr);
    }

  return(0);
  }

/***************************************************************************************************************************/
